//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     garg.h
// Purpose:    Argument (list) of ground literal
// Last Change:11.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	GARG_INCLUDED
#define GARG_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef CNST_INCLUDED
#include "cnst.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// GARG_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
static const long GARG_MAGIC =	0x47415200L;	// 'GAR\0'
#endif

//=============================================================================
// Class for Lists of Terms:
//=============================================================================

class garg_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor:
	garg_c(cnst_t cnst);

	// Destructor:
	~garg_c(void);

	// Get the constant in this argument:
	inline cnst_t	get_cnst(void) const
		{ CHECK(valid(), "garg_c::get_cnst: Invalid Object");
		  return(Cnst); }

	// Get the link to the next argument:
	inline garg_c*	get_next(void) const
		{ CHECK(valid(), "garg_c::get_next: Invalid Object");
		  return(Next); }

	// Set link to next argument (only possible if this is the last one):
	inline void	set_next(garg_c* next)
		{ CHECK(valid(), "garg_c::set_next: Invalid Object");
		  CHECK(next != 0, "garg_c::set_next: Next is NIL");
		  CHECK(next->valid(), "garg_c::set_next: Next is invalid");
		  CHECK(Next == 0, "garg_c::set_next: Next-link already set");
		  Next = next; }

	// Print list of argument terms:
	void print_list(void) const;

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "GARG_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Cnst: First term in the list.
	cnst_t	Cnst;

	// Next: Rest of the list (i.e. this list except the first term).
	garg_c *Next;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef garg_c *garg_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

